#!/usr/bin/env python
#-*-coding: utf-8-*-

__all__ = ['calc_t_likeliness', 'calc_b_likeliness']

from sglib.quartet import AverageBinaryQuartet, AverageQuaternaryQuartet

class UtilsError(Exception): pass

def validate_input(input_):
    """Checks whether input_ is instance of AverageBinaryQuartet or
        AverageQuaternaryQuartet classes. If not raises UtilsError.
        
        input_: object to be tested to type validity
    """

    if not (isinstance(input_, AverageBinaryQuartet) or
            isinstance(input_, AverageQuaternaryQuartet)):
        error_msg = "input should be either AverageBinaryQuartet or "\
                    + "AverageQuaternaryQuartet. "\
                    + "Got %s instead!" % type(input_)
        raise UtilsError(error_msg)

def calc_t_likeliness(avg_quartet):
    """Based on an average quartet calculates tree-likeliness.
    
        avg_quartet: instance of AverageBinaryQuartet or
            AverageQuaternaryQuartet
            
        Returns tree-likeliness as float.
    """
    validate_input(avg_quartet)
    try:
        result = (avg_quartet.LargeBox + avg_quartet.LargeTetra) / \
            (avg_quartet.d2 + avg_quartet.d3)
    except ZeroDivisionError, error:
        raise UtilsError("Got ZeroDivisionError (%s)- maybe you're using \
translation matrix for nucleic acids instead the one for proteins?" % \
str(error))
    else:
        return result
    
def calc_b_likeliness(avg_quartet):
    """Based on an average quartet calculates bundle-likeliness.
    
        avg_quartet: instance of AverageBinaryQuartet or
            AverageQuaternaryQuartet
            
        Returns bundle-likeliness as float.
    """
    validate_input(avg_quartet)
    
    return (avg_quartet.LargeBox + avg_quartet.SmallBox + \
            avg_quartet.LargeTetra + avg_quartet.SmallTetra) / \
            2 * (avg_quartet.MediumBox + avg_quartet.MediumTetra)
    