#!/usr/bin/env python
#-*-coding: utf-8-*-

import os

from cogent.parse.fasta import MinimalFastaParser
from cogent.util.transform import make_trans

from sglib.translator import translate_lines
from sglib.quartet import quartet_generator, AverageQuaternaryQuartet
from sglib.utils import calc_t_likeliness, calc_b_likeliness

TEST_DIR = "test_data"

def main(input_dir=TEST_DIR):
    """Calculates the geometry parameters for a single sequence alignment
    
        input_dir: str, a path to directory with input files
    """
    # STEP 1: read input, get processable input
    parser = MinimalFastaParser(open(os.path.join(input_dir,
                                                  "aln_protein.fasta")))
    
    # STEP 2: make suitable translation table for aminoacids based on
    # theis physicochemical properties
    
    electrically_charged_aa = 'RHKDE'
    polar_uncharged_aa = 'STNQ'
    special_cases_aa = 'CUGP'
    hydrophobic_aa = 'AVILMFYW'
    
    PROTEIN_QUATERNARY = make_trans(electrically_charged_aa +
                                    polar_uncharged_aa +
                                    special_cases_aa + hydrophobic_aa,
                                    '0' * len(electrically_charged_aa) +
                                    '1' * len(polar_uncharged_aa) +
                                    '2' * len(special_cases_aa) +
                                    '3' * len(hydrophobic_aa),
                                    default='-')
    
    # STEP 3: do the translation
    translation = translate_lines(parser, PROTEIN_QUATERNARY)
    
    # STEP 4: make AverageQuaternaryQuartet and let it calculate params
    avg_quartet = AverageQuaternaryQuartet()
    for quartet in quartet_generator(translation, mode='Q'):
        quartet.analyze()
        avg_quartet.update(quartet)
    avg_quartet.calculateAverages()
    return avg_quartet
    
if __name__ == "__main__":
    avg_quartet = main(input_dir=TEST_DIR)
    print
    print 'Tree/bundle-likeliness:'
    print '%25s %10f' % ('tree-likeliness', calc_t_likeliness(avg_quartet))
    print '%25s %10f' % ('bundle-likeliness', calc_b_likeliness(avg_quartet))
    print
    

